# OSeMOSYS_2013_05_10
# 
# Open Source energy MOdeling SYStem
#
# Main changes to previous version OSeMOSYS_2013_04_30
#		- Re-ordered the arguments of parameters, variables and constraints to be compatible for use with the ANSWER interface which is currently being developed by Noble-Soft Systems Pty Ltd. 
#		Henceforth, this reordered indexing will be applied to all future versions of the OSeMOSYS model.
#
# ============================================================================
#
#    Copyright [2010-2013] [OSeMOSYS Forum steering committee see: www.osemosys.org]
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
# ============================================================================
#
#  To run OSeMOSYS, enter the following line into your command prompt after replacing FILEPATH & YOURDATAFILE with your folder structure and data file name: 
#
#  C:\...FILEPATH...\glpsol -m C:\...FILEPATH...\OSeMOSYS_2013_05_10.mod -d C:\...FILEPATH...\YOURDATAFILE.dat -o C:\...FILEPATH...\Results.txt
#
#  Alternatively, install GUSEK (http://gusek.sourceforge.net/gusek.html) and run the model within this integrated development environment (IDE). 
#  To do so, open the .dat file and select "Use External .dat file" from the Options menu. Then change to the model file and select the "Go" icon or press F5.
#


# Inclusion of a behavioral approach, 
# an information campaign and a subvention capaign
# done in 2017 by Francesco Moresino.
# The modifications are tagged with the comment "behavioral approach"
#





#              			#########################################
######################			Model Definition				#############
#              			#########################################
#
###############
#    Sets     #
############### 
# 
set YEAR; 
param first_year:= min{y in YEAR} y;
set TECHNOLOGY;
set TIMESLICE;
set FUEL;
set EMISSION;
set MODE_OF_OPERATION;
set REGION;
set SEASON;
set DAYTYPE;
set DAILYTIMEBRACKET;
set FLEXIBLEDEMANDTYPE; 
set STORAGE;
#
#####################
#    Parameters     #
#####################
#	
########			Global 						#############
#
param YearSplit{l in TIMESLICE, y in YEAR};
param DiscountRate{r in REGION, t in TECHNOLOGY};
param DaySplit{lh in DAILYTIMEBRACKET, y in YEAR};
param Conversionls{l in TIMESLICE, ls in SEASON};
param Conversionld{l in TIMESLICE, ld in DAYTYPE};
param Conversionlh{l in TIMESLICE, lh in DAILYTIMEBRACKET};
param DaysInDayType{ls in SEASON, ld in DAYTYPE, y in YEAR};
param TradeRoute{r in REGION, rr in REGION, f in FUEL, y in YEAR};
param DepreciationMethod{r in REGION};
#
########			Demands 					#############
#
param SpecifiedAnnualDemand{r in REGION, f in FUEL, y in YEAR}; 
param SpecifiedDemandProfile{r in REGION, f in FUEL, l in TIMESLICE, y in YEAR};
param AccumulatedAnnualDemand{r in REGION, f in FUEL, y in YEAR};
#
#########			Performance					#############
#
param CapacityToActivityUnit{r in REGION, t in TECHNOLOGY};
param TechWithCapacityNeededToMeetPeakTS{r in REGION, t in TECHNOLOGY};
param CapacityFactor{r in REGION, t in TECHNOLOGY, l in TIMESLICE, y in YEAR};
param AvailabilityFactor{r in REGION, t in TECHNOLOGY, y in YEAR};
param OperationalLife{r in REGION, t in TECHNOLOGY};
param ResidualCapacity{r in REGION, t in TECHNOLOGY, y in YEAR};
param InputActivityRatio{r in REGION, t in TECHNOLOGY, f in FUEL, m in MODE_OF_OPERATION, y in YEAR};
param OutputActivityRatio{r in REGION, t in TECHNOLOGY, f in FUEL, m in MODE_OF_OPERATION, y in YEAR};
#
#########			Technology Costs			#############
#
param CapitalCost{r in REGION, t in TECHNOLOGY, y in YEAR};
param VariableCost{r in REGION, t in TECHNOLOGY, m in MODE_OF_OPERATION, y in YEAR};
param FixedCost{r in REGION, t in TECHNOLOGY, y in YEAR};
#
#########           		Storage                 		#############
#
param TechnologyToStorage{r in REGION, t in TECHNOLOGY, s in STORAGE, m in MODE_OF_OPERATION};
param TechnologyFromStorage{r in REGION, t in TECHNOLOGY, s in STORAGE, m in MODE_OF_OPERATION};
param StorageLevelStart{r in REGION, s in STORAGE};
param StorageMaxChargeRate{r in REGION, s in STORAGE};
param StorageMaxDischargeRate{r in REGION, s in STORAGE};
param MinStorageCharge{r in REGION, s in STORAGE, y in YEAR};
param OperationalLifeStorage{r in REGION, s in STORAGE};
param CapitalCostStorage{r in REGION, s in STORAGE, y in YEAR};
param DiscountRateStorage{r in REGION, s in STORAGE};
param ResidualStorageCapacity{r in REGION, s in STORAGE, y in YEAR};
#
#########			Capacity Constraints		#############
#
param CapacityOfOneTechnologyUnit{r in REGION, t in TECHNOLOGY, y in YEAR};
param TotalAnnualMaxCapacity{r in REGION, t in TECHNOLOGY, y in YEAR};
param TotalAnnualMinCapacity{r in REGION, t in TECHNOLOGY, y in YEAR};
#
#########			Investment Constraints		#############
#
param TotalAnnualMaxCapacityInvestment{r in REGION, t in TECHNOLOGY, y in YEAR};
param TotalAnnualMinCapacityInvestment{r in REGION, t in TECHNOLOGY, y in YEAR};
#
#########			Activity Constraints		#############
#
param TotalTechnologyAnnualActivityUpperLimit{r in REGION, t in TECHNOLOGY, y in YEAR};
param TotalTechnologyAnnualActivityLowerLimit{r in REGION, t in TECHNOLOGY, y in YEAR};
param TotalTechnologyModelPeriodActivityUpperLimit{r in REGION, t in TECHNOLOGY};
param TotalTechnologyModelPeriodActivityLowerLimit{r in REGION, t in TECHNOLOGY};
#
#########			Reserve Margin				############# 
#
param ReserveMarginTagTechnology{r in REGION, t in TECHNOLOGY, y in YEAR}; 
param ReserveMarginTagFuel{r in REGION, f in FUEL, y in YEAR};
param ReserveMargin{r in REGION, y in YEAR};
#
#########			RE Generation Target		############# 
#
param RETagTechnology{r in REGION, t in TECHNOLOGY, y in YEAR};
param RETagFuel{r in REGION, f in FUEL, y in YEAR}; 
param REMinProductionTarget{r in REGION, y in YEAR};
#
#########			Emissions & Penalties		#############
#
param EmissionActivityRatio{r in REGION, t in TECHNOLOGY, e in EMISSION, m in MODE_OF_OPERATION, y in YEAR};
param EmissionsPenalty{r in REGION, e in EMISSION, y in YEAR};
param AnnualExogenousEmission{r in REGION, e in EMISSION, y in YEAR};
param AnnualEmissionLimit{r in REGION, e in EMISSION, y in YEAR};
param ModelPeriodExogenousEmission{r in REGION, e in EMISSION};
param ModelPeriodEmissionLimit{r in REGION, e in EMISSION};
#
######################
#   Model Variables  #
######################
#
########			Demands 					#############
#
var RateOfDemand{r in REGION, l in TIMESLICE, f in FUEL, y in YEAR}>= 0;
var Demand{r in REGION, l in TIMESLICE, f in FUEL, y in YEAR}>= 0;
#
########     		Storage                 		#############
#
var RateOfStorageCharge{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR};
var RateOfStorageDischarge{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR};
var NetChargeWithinYear{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR};
var NetChargeWithinDay{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR};
var StorageLevelYearStart{r in REGION, s in STORAGE, y in YEAR} >=0;
var StorageLevelYearFinish{r in REGION, s in STORAGE, y in YEAR} >=0;
var StorageLevelSeasonStart{r in REGION, s in STORAGE, ls in SEASON, y in YEAR} >=0;
var StorageLevelDayTypeStart{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, y in YEAR} >=0;
var StorageLevelDayTypeFinish{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, y in YEAR} >=0;
var StorageLowerLimit{r in REGION, s in STORAGE, y in YEAR}>=0;
var StorageUpperLimit{r in REGION, s in STORAGE, y in YEAR} >=0;
var AccumulatedNewStorageCapacity{r in REGION, s in STORAGE, y in YEAR} >=0;
var NewStorageCapacity{r in REGION, s in STORAGE, y in YEAR} >=0;
var CapitalInvestmentStorage{r in REGION, s in STORAGE, y in YEAR} >=0;
var DiscountedCapitalInvestmentStorage{r in REGION, s in STORAGE, y in YEAR} >=0;
var SalvageValueStorage{r in REGION, s in STORAGE, y in YEAR} >=0;
var DiscountedSalvageValueStorage{r in REGION, s in STORAGE, y in YEAR} >=0;
var TotalDiscountedStorageCost{r in REGION, s in STORAGE, y in YEAR} >=0;
#
#########		    Capacity Variables 			############# 
#
var NumberOfNewTechnologyUnits{r in REGION, t in TECHNOLOGY, y in YEAR} >= 0,integer;
var NewCapacity{r in REGION, t in TECHNOLOGY, y in YEAR} >= 0;
var AccumulatedNewCapacity{r in REGION, t in TECHNOLOGY, y in YEAR} >= 0;
var TotalCapacityAnnual{r in REGION, t in TECHNOLOGY, y in YEAR}>= 0;
#
#########		    Activity Variables 			#############
#
var RateOfActivity{r in REGION, l in TIMESLICE, t in TECHNOLOGY, m in MODE_OF_OPERATION, y in YEAR} >= 0; 
var RateOfTotalActivity{r in REGION, t in TECHNOLOGY, l in TIMESLICE, y in YEAR} >= 0;
var TotalTechnologyAnnualActivity{r in REGION, t in TECHNOLOGY, y in YEAR} >= 0;
var TotalAnnualTechnologyActivityByMode{r in REGION, t in TECHNOLOGY, m in MODE_OF_OPERATION, y in YEAR}>=0;
var RateOfProductionByTechnologyByMode{r in REGION, l in TIMESLICE, t in TECHNOLOGY, m in MODE_OF_OPERATION, f in FUEL, y in YEAR}>= 0;
var RateOfProductionByTechnology{r in REGION, l in TIMESLICE, t in TECHNOLOGY, f in FUEL, y in YEAR}>= 0;
var ProductionByTechnology{r in REGION, l in TIMESLICE, t in TECHNOLOGY, f in FUEL, y in YEAR}>= 0;
var ProductionByTechnologyAnnual{r in REGION, t in TECHNOLOGY, f in FUEL, y in YEAR}>= 0;
var RateOfProduction{r in REGION, l in TIMESLICE, f in FUEL, y in YEAR} >= 0;
var Production{r in REGION, l in TIMESLICE, f in FUEL, y in YEAR} >= 0;
var RateOfUseByTechnologyByMode{r in REGION, l in TIMESLICE, t in TECHNOLOGY, m in MODE_OF_OPERATION, f in FUEL, y in YEAR}>= 0;
var RateOfUseByTechnology{r in REGION, l in TIMESLICE, t in TECHNOLOGY, f in FUEL, y in YEAR} >= 0;
var UseByTechnologyAnnual{r in REGION, t in TECHNOLOGY, f in FUEL, y in YEAR}>= 0;
var RateOfUse{r in REGION, l in TIMESLICE, f in FUEL, y in YEAR}>= 0;
var UseByTechnology{r in REGION, l in TIMESLICE, t in TECHNOLOGY, f in FUEL, y in YEAR}>= 0;
var Use{r in REGION, l in TIMESLICE, f in FUEL, y in YEAR}>= 0;
var Trade{r in REGION, rr in REGION, l in TIMESLICE, f in FUEL, y in YEAR};
var TradeAnnual{r in REGION, rr in REGION, f in FUEL, y in YEAR};
#
var ProductionAnnual{r in REGION, f in FUEL, y in YEAR}>= 0;
var UseAnnual{r in REGION, f in FUEL, y in YEAR}>= 0;
#
#########		    Costing Variables 			#############
#
var CapitalInvestment{r in REGION, t in TECHNOLOGY, y in YEAR}>= 0;
var DiscountedCapitalInvestment{r in REGION, t in TECHNOLOGY, y in YEAR}>= 0;
#
var SalvageValue{r in REGION, t in TECHNOLOGY, y in YEAR}>= 0;
var DiscountedSalvageValue{r in REGION, t in TECHNOLOGY, y in YEAR}>= 0;
var OperatingCost{r in REGION, t in TECHNOLOGY, y in YEAR}>= 0;
var DiscountedOperatingCost{r in REGION, t in TECHNOLOGY, y in YEAR}>= 0;
#
var AnnualVariableOperatingCost{r in REGION, t in TECHNOLOGY, y in YEAR}>= 0;
var AnnualFixedOperatingCost{r in REGION, t in TECHNOLOGY, y in YEAR}>= 0;
var VariableOperatingCost{r in REGION, t in TECHNOLOGY, l in TIMESLICE, y in YEAR}>= 0;
#
var TotalDiscountedCostByTechnology{r in REGION, t in TECHNOLOGY, y in YEAR}>= 0;
var TotalDiscountedCost{r in REGION, y in YEAR}>= 0;
#
var ModelPeriodCostByRegion{r in REGION} >= 0;
#
#########			Reserve Margin				#############
#
var TotalCapacityInReserveMargin{r in REGION, y in YEAR}>= 0;
var DemandNeedingReserveMargin{r in REGION,l in TIMESLICE, y in YEAR}>= 0;
#
#########			RE Gen Target				#############
#
var TotalREProductionAnnual{r in REGION, y in YEAR};
var RETotalDemandOfTargetFuelAnnual{r in REGION, y in YEAR};
#
var TotalTechnologyModelPeriodActivity{r in REGION, t in TECHNOLOGY};
#
#########			Emissions					#############
#
var AnnualTechnologyEmissionByMode{r in REGION, t in TECHNOLOGY, e in EMISSION, m in MODE_OF_OPERATION, y in YEAR}>= 0;
var AnnualTechnologyEmission{r in REGION, t in TECHNOLOGY, e in EMISSION, y in YEAR}>= 0;
var AnnualTechnologyEmissionPenaltyByEmission{r in REGION, t in TECHNOLOGY, e in EMISSION, y in YEAR}>= 0;
var AnnualTechnologyEmissionsPenalty{r in REGION, t in TECHNOLOGY, y in YEAR}>= 0;
var DiscountedTechnologyEmissionsPenalty{r in REGION, t in TECHNOLOGY, y in YEAR}>= 0;
var AnnualEmissions{r in REGION, e in EMISSION, y in YEAR}>= 0;
var ModelPeriodEmissions{r in REGION, e in EMISSION}>= 0;
#
#	table data IN "CSV" "data.csv": s <- [FROM,TO], d~DISTANCE, c~COST;
#	table capacity IN "CSV" "SpecifiedAnnualDemand.csv": [YEAR, FUEL, REGION], SpecifiedAnnualDemand~ColumnNameInCSVSheet;





#########################################
# Policy (behavioral approach)
#########################################

set CAMPAIGN;  
set SUBVENTION;

param COST_CAMPAIGN{c in CAMPAIGN};
param ACCEPTANCE_SUBVENTION; # between 0 and 1, is the 	#percentage of the subvention cost that is accounted as 	#acceptance costs

var campaign{c in CAMPAIGN} binary;
var subvention{s in SUBVENTION} binary;
var cost_subvention;
var LED_bought; # million LED bulbs bought

#########################################
# Share of choice (behavioral approach)
#########################################

param BIGM;
param SHARE{c in CAMPAIGN, s in SUBVENTION};




#
######################
# Objective Function #
######################
#
minimize cost: (sum{r in REGION, y in YEAR} TotalDiscountedCost[r,y]) 
+ (sum{c in CAMPAIGN} COST_CAMPAIGN[c] * campaign[c])    # behavioral approach
+ cost_subvention; # behavioral approach




#
#####################
# Policy   (behavioral approach)  
#####################


subject to E_LED_bought: 
LED_bought = 
(sum {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL2'}NewCapacity[r,t,y])*278/14 ;
# million LED bought
# 1PJ= 278'000'000  kWh; 
# 1 bulb =14 W; 1 bulb 1000 hours a year= 14 kWh




subject to E1_cost_subvention{s in SUBVENTION}:
cost_subvention <= 
ACCEPTANCE_SUBVENTION *LED_bought*s + BIGM* (1-subvention[s]);
 
subject to E2_cost_subvention{s in SUBVENTION}:
cost_subvention >= 
ACCEPTANCE_SUBVENTION *LED_bought*s - BIGM* (1-subvention[s]);




subject to normalisation1:
sum {c in CAMPAIGN} campaign[c]=1;


subject to normalisation2:
sum {s in SUBVENTION} subvention[s]=1;


#
#####################
# Share of choice  (behavioral approach)    #
#####################



# y<first_year+ OperationalLife[r,"RL1"] 


subject to share_a1{r in REGION, y in YEAR, c in CAMPAIGN, s in SUBVENTION: y<first_year+ OperationalLife[r,"RL1"]}:
NewCapacity[r,"RL1",y] <=
NewCapacity[r,"RL2",y]*((1-SHARE[c,s])/SHARE[c,s])
+ BIGM *(1-subvention[s])+ BIGM *(1- campaign[c]) ;


subject to share_1b1{r in REGION, y in YEAR, c in CAMPAIGN, s in SUBVENTION: y<first_year+ OperationalLife[r,"RL1"]}:
NewCapacity[r,"RL1",y]>=
NewCapacity[r,"RL2",y]*((1-SHARE[c,s])/SHARE[c,s])
- BIGM *(1-subvention[s])- BIGM *(1-campaign[c]) ;


#	first_year+ OperationalLife[r,"RL1"] 
# 	<= y <
#	first_year+ OperationalLife[r,"RL2"]  


subject to share_a2{r in REGION, y in YEAR, c in CAMPAIGN, s in SUBVENTION: 
y>=first_year+ OperationalLife[r,"RL1"] and 
y< first_year+ OperationalLife[r,"RL2"] }:
NewCapacity[r,"RL1",y] - 
NewCapacity[r,"RL1",y-OperationalLife[r,"RL1"]] <=
NewCapacity[r,"RL2",y]*((1-SHARE[c,s])/SHARE[c,s])
+ BIGM *(1-subvention[s])+ BIGM *(1-campaign[c]) ;

subject to share_b2{r in REGION, y in YEAR, c in CAMPAIGN, s in SUBVENTION: 
y>=first_year+ OperationalLife[r,"RL1"] and 
y< first_year+ OperationalLife[r,"RL2"] }:
NewCapacity[r,"RL1",y] - 
NewCapacity[r,"RL1",y-OperationalLife[r,"RL1"]] >=
NewCapacity[r,"RL2",y]*((1-SHARE[c,s])/SHARE[c,s])
- BIGM *(1-subvention[s])- BIGM *(1-campaign[c]) ;



# y>= first_year+ OperationalLife[r,"RL2"]}:

subject to share_a3{r in REGION, y in YEAR, c in CAMPAIGN, s in SUBVENTION: 
y>= first_year+ OperationalLife[r,"RL2"]}:
NewCapacity[r,"RL1",y] - 
NewCapacity[r,"RL1",y-OperationalLife[r,"RL1"]] <=
(NewCapacity[r,"RL2",y]
-NewCapacity[r,"RL2",y-OperationalLife[r,"RL2"]])
*((1-SHARE[c,s])/SHARE[c,s])
+ BIGM *(1-subvention[s])+ BIGM *(1-campaign[c]) ;

subject to share_b3{r in REGION, y in YEAR, c in CAMPAIGN, s in SUBVENTION: 
y>= first_year+ OperationalLife[r,"RL2"]}:
NewCapacity[r,"RL1",y] - 
NewCapacity[r,"RL1",y-OperationalLife[r,"RL1"]] >=
(NewCapacity[r,"RL2",y]
-NewCapacity[r,"RL2",y-OperationalLife[r,"RL2"]])
*((1-SHARE[c,s])/SHARE[c,s])
- BIGM *(1-subvention[s])- BIGM *(1-campaign[c]) ;




#
#####################
# Constraints       #
#####################
#
s.t. EQ_SpecifiedDemand{r in REGION, l in TIMESLICE, f in FUEL, y in YEAR}: SpecifiedAnnualDemand[r,f,y]*SpecifiedDemandProfile[r,f,l,y] / YearSplit[l,y]=RateOfDemand[r,l,f,y];
#
#########       	Capacity Adequacy A	     	#############
#
s.t. CAa1_TotalNewCapacity{r in REGION, t in TECHNOLOGY, y in YEAR}:AccumulatedNewCapacity[r,t,y] = sum{yy in YEAR: y-yy < OperationalLife[r,t] && y-yy>=0} 
								if CapacityOfOneTechnologyUnit[r,t,y]=0 then NewCapacity[r,t,yy]
								else CapacityOfOneTechnologyUnit[r,t,yy]*NumberOfNewTechnologyUnits[r,t,yy];
s.t. CAa2_TotalAnnualCapacity{r in REGION, t in TECHNOLOGY, y in YEAR}: AccumulatedNewCapacity[r,t,y]+ ResidualCapacity[r,t,y] = TotalCapacityAnnual[r,t,y];
s.t. CAa3_TotalActivityOfEachTechnology{r in REGION, t in TECHNOLOGY, l in TIMESLICE, y in YEAR}: sum{m in MODE_OF_OPERATION} RateOfActivity[r,l,t,m,y] = RateOfTotalActivity[r,t,l,y];
s.t. CAa4_Constraint_Capacity{r in REGION, l in TIMESLICE, t in TECHNOLOGY, y in YEAR: TechWithCapacityNeededToMeetPeakTS[r,t]<>0}: RateOfTotalActivity[r,t,l,y] <= TotalCapacityAnnual[r,t,y] * CapacityFactor[r,t,l,y]*CapacityToActivityUnit[r,t];
#
# Note that the PlannedMaintenance equation below ensures that all other technologies have a capacity great enough to at least meet the annual average.
#
#########       	Capacity Adequacy B		 	#############
#
s.t. CAb1_PlannedMaintenance{r in REGION, t in TECHNOLOGY, y in YEAR}: sum{l in TIMESLICE} RateOfTotalActivity[r,t,l,y]*YearSplit[l,y] <= sum{l in TIMESLICE} (TotalCapacityAnnual[r,t,y]*CapacityFactor[r,t,l,y]*YearSplit[l,y])* AvailabilityFactor[r,t,y]*CapacityToActivityUnit[r,t];
#
#########	        Energy Balance A    	 	#############
#
s.t. EBa1_RateOfFuelProduction1{r in REGION, l in TIMESLICE, f in FUEL, t in TECHNOLOGY, m in MODE_OF_OPERATION, y in YEAR: OutputActivityRatio[r,t,f,m,y] <>0}:  RateOfActivity[r,l,t,m,y]*OutputActivityRatio[r,t,f,m,y]  = RateOfProductionByTechnologyByMode[r,l,t,m,f,y];
s.t. EBa2_RateOfFuelProduction2{r in REGION, l in TIMESLICE, f in FUEL, t in TECHNOLOGY, y in YEAR}: sum{m in MODE_OF_OPERATION: OutputActivityRatio[r,t,f,m,y] <>0} RateOfProductionByTechnologyByMode[r,l,t,m,f,y] = RateOfProductionByTechnology[r,l,t,f,y] ;
s.t. EBa3_RateOfFuelProduction3{r in REGION, l in TIMESLICE, f in FUEL, y in YEAR}: sum{t in TECHNOLOGY} RateOfProductionByTechnology[r,l,t,f,y]  =  RateOfProduction[r,l,f,y];
s.t. EBa4_RateOfFuelUse1{r in REGION, l in TIMESLICE, f in FUEL, t in TECHNOLOGY, m in MODE_OF_OPERATION, y in YEAR: InputActivityRatio[r,t,f,m,y]<>0}: RateOfActivity[r,l,t,m,y]*InputActivityRatio[r,t,f,m,y]  = RateOfUseByTechnologyByMode[r,l,t,m,f,y];
s.t. EBa5_RateOfFuelUse2{r in REGION, l in TIMESLICE, f in FUEL, t in TECHNOLOGY, y in YEAR}: sum{m in MODE_OF_OPERATION: InputActivityRatio[r,t,f,m,y]<>0} RateOfUseByTechnologyByMode[r,l,t,m,f,y] = RateOfUseByTechnology[r,l,t,f,y];
s.t. EBa6_RateOfFuelUse3{r in REGION, l in TIMESLICE, f in FUEL, y in YEAR}: sum{t in TECHNOLOGY} RateOfUseByTechnology[r,l,t,f,y]  = RateOfUse[r,l,f,y];
s.t. EBa7_EnergyBalanceEachTS1{r in REGION, l in TIMESLICE, f in FUEL, y in YEAR}: RateOfProduction[r,l,f,y]*YearSplit[l,y] = Production[r,l,f,y];
s.t. EBa8_EnergyBalanceEachTS2{r in REGION, l in TIMESLICE, f in FUEL, y in YEAR}: RateOfUse[r,l,f,y]*YearSplit[l,y] = Use[r,l,f,y];
s.t. EBa9_EnergyBalanceEachTS3{r in REGION, l in TIMESLICE, f in FUEL, y in YEAR}: RateOfDemand[r,l,f,y]*YearSplit[l,y] = Demand[r,l,f,y];
s.t. EBa10_EnergyBalanceEachTS4{r in REGION, rr in REGION, l in TIMESLICE, f in FUEL, y in YEAR}: Trade[r,rr,l,f,y] = -Trade[rr,r,l,f,y];
s.t. EBa11_EnergyBalanceEachTS5{r in REGION, l in TIMESLICE, f in FUEL, y in YEAR}: Production[r,l,f,y] >= Demand[r,l,f,y] + Use[r,l,f,y] + sum{rr in REGION} Trade[r,rr,l,f,y]*TradeRoute[r,rr,f,y];
#
#########        	Energy Balance B		 	#############
#
s.t. EBb1_EnergyBalanceEachYear1{r in REGION, f in FUEL, y in YEAR}: sum{l in TIMESLICE} Production[r,l,f,y] = ProductionAnnual[r,f,y];
s.t. EBb2_EnergyBalanceEachYear2{r in REGION, f in FUEL, y in YEAR}: sum{l in TIMESLICE} Use[r,l,f,y] = UseAnnual[r,f,y];
s.t. EBb3_EnergyBalanceEachYear3{r in REGION, rr in REGION, f in FUEL, y in YEAR}: sum{l in TIMESLICE} Trade[r,rr,l,f,y] = TradeAnnual[r,rr,f,y];
s.t. EBb4_EnergyBalanceEachYear4{r in REGION, f in FUEL, y in YEAR}: ProductionAnnual[r,f,y] >= UseAnnual[r,f,y] + sum{rr in REGION} TradeAnnual[r,rr,f,y]*TradeRoute[r,rr,f,y] + AccumulatedAnnualDemand[r,f,y];
#
#########        	Accounting Technology Production/Use	#############
#
s.t. Acc1_FuelProductionByTechnology{r in REGION, l in TIMESLICE, t in TECHNOLOGY, f in FUEL, y in YEAR}: RateOfProductionByTechnology[r,l,t,f,y] * YearSplit[l,y] = ProductionByTechnology[r,l,t,f,y];
s.t. Acc2_FuelUseByTechnology{r in REGION, l in TIMESLICE, t in TECHNOLOGY, f in FUEL, y in YEAR}: RateOfUseByTechnology[r,l,t,f,y] * YearSplit[l,y] = UseByTechnology[r,l,t,f,y];
s.t. Acc3_AverageAnnualRateOfActivity{r in REGION, t in TECHNOLOGY, m in MODE_OF_OPERATION, y in YEAR}: sum{l in TIMESLICE} RateOfActivity[r,l,t,m,y]*YearSplit[l,y] = TotalAnnualTechnologyActivityByMode[r,t,m,y];
s.t. Acc4_ModelPeriodCostByRegion{r in REGION}:sum{y in YEAR}TotalDiscountedCost[r,y] = ModelPeriodCostByRegion[r];
#
#########        	Storage Equations			#############
#
s.t. S1_RateOfStorageCharge{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR}: sum{t in TECHNOLOGY, m in MODE_OF_OPERATION, l in TIMESLICE:TechnologyToStorage[r,t,s,m]>0} RateOfActivity[r,l,t,m,y] * TechnologyToStorage[r,t,s,m] * Conversionls[l,ls] * Conversionld[l,ld] * Conversionlh[l,lh] = RateOfStorageCharge[r,s,ls,ld,lh,y];
s.t. S2_RateOfStorageDischarge{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR}: sum{t in TECHNOLOGY, m in MODE_OF_OPERATION, l in TIMESLICE:TechnologyFromStorage[r,t,s,m]>0} RateOfActivity[r,l,t,m,y] * TechnologyFromStorage[r,t,s,m] * Conversionls[l,ls] * Conversionld[l,ld] * Conversionlh[l,lh] = RateOfStorageDischarge[r,s,ls,ld,lh,y];
s.t. S3_NetChargeWithinYear{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR}: sum{l in TIMESLICE:Conversionls[l,ls]>0&&Conversionld[l,ld]>0&&Conversionlh[l,lh]>0}  (RateOfStorageCharge[r,s,ls,ld,lh,y] - RateOfStorageDischarge[r,s,ls,ld,lh,y]) * YearSplit[l,y] * Conversionls[l,ls] * Conversionld[l,ld] * Conversionlh[l,lh] = NetChargeWithinYear[r,s,ls,ld,lh,y];
s.t. S4_NetChargeWithinDay{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR}: (RateOfStorageCharge[r,s,ls,ld,lh,y] - RateOfStorageDischarge[r,s,ls,ld,lh,y]) * DaySplit[lh,y] = NetChargeWithinDay[r,s,ls,ld,lh,y];
s.t. S5_and_S6_StorageLevelYearStart{r in REGION, s in STORAGE, y in YEAR}: if y = min{yy in YEAR} min(yy) then StorageLevelStart[r,s] 
																	else StorageLevelYearStart[r,s,y-1] + sum{ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET} NetChargeWithinYear[r,s,ls,ld,lh,y-1]
																	= StorageLevelYearStart[r,s,y];
s.t. S7_and_S8_StorageLevelYearFinish{r in REGION, s in STORAGE, y in YEAR}: if y < max{yy in YEAR} max(yy) then StorageLevelYearStart[r,s,y+1]
																	else StorageLevelYearStart[r,s,y] + sum{ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET} NetChargeWithinYear[r,s,ls,ld,lh,y] 
																	= StorageLevelYearFinish[r,s,y];	
s.t. S9_and_S10_StorageLevelSeasonStart{r in REGION, s in STORAGE, ls in SEASON, y in YEAR}: if ls = min{lsls in SEASON} min(lsls) then StorageLevelYearStart[r,s,y] 
																	else StorageLevelSeasonStart[r,s,ls-1,y] + sum{ld in DAYTYPE, lh in DAILYTIMEBRACKET} NetChargeWithinYear[r,s,ls-1,ld,lh,y] 
																	= StorageLevelSeasonStart[r,s,ls,y];
s.t. S11_and_S12_StorageLevelDayTypeStart{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, y in YEAR}: if ld = min{ldld in DAYTYPE} min(ldld) then StorageLevelSeasonStart[r,s,ls,y] 
																	else StorageLevelDayTypeStart[r,s,ls,ld-1,y] + sum{lh in DAILYTIMEBRACKET} NetChargeWithinDay[r,s,ls,ld-1,lh,y] * DaysInDayType[ls,ld-1,y]
																	= StorageLevelDayTypeStart[r,s,ls,ld,y];
s.t. S13_and_S14_and_S15_StorageLevelDayTypeFinish{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, y in YEAR}:	if ls = max{lsls in SEASON} max(lsls) && ld = max{ldld in DAYTYPE} max(ldld) then StorageLevelYearFinish[r,s,y] 
																	else if ld = max{ldld in DAYTYPE} max(ldld) then StorageLevelSeasonStart[r,s,ls+1,y]
																	else StorageLevelDayTypeFinish[r,s,ls,ld+1,y] - sum{lh in DAILYTIMEBRACKET} NetChargeWithinDay[r,s,ls,ld+1,lh,y] * DaysInDayType[ls,ld+1,y]
																	= StorageLevelDayTypeFinish[r,s,ls,ld,y];	
#
##########		Storage Constraints				#############
#																	
s.t. SC1_LowerLimit_BeginningOfDailyTimeBracketOfFirstInstanceOfDayTypeInFirstWeekConstraint{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR}: 0 <= (StorageLevelDayTypeStart[r,s,ls,ld,y]+sum{lhlh in DAILYTIMEBRACKET:lh-lhlh>0} NetChargeWithinDay[r,s,ls,ld,lhlh,y])-StorageLowerLimit[r,s,y];								
s.t. SC1_UpperLimit_BeginningOfDailyTimeBracketOfFirstInstanceOfDayTypeInFirstWeekConstraint{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR}: (StorageLevelDayTypeStart[r,s,ls,ld,y]+sum{lhlh in DAILYTIMEBRACKET:lh-lhlh>0} NetChargeWithinDay[r,s,ls,ld,lhlh,y])-StorageUpperLimit[r,s,y] <= 0;								
s.t. SC2_LowerLimit_EndOfDailyTimeBracketOfLastInstanceOfDayTypeInFirstWeekConstraint{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR}: 0 <= if ld > min{ldld in DAYTYPE} min(ldld) then (StorageLevelDayTypeStart[r,s,ls,ld,y]-sum{lhlh in DAILYTIMEBRACKET:lh-lhlh<0} NetChargeWithinDay[r,s,ls,ld-1,lhlh,y])-StorageLowerLimit[r,s,y];																										
s.t. SC2_UpperLimit_EndOfDailyTimeBracketOfLastInstanceOfDayTypeInFirstWeekConstraint{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR}: if ld > min{ldld in DAYTYPE} min(ldld) then (StorageLevelDayTypeStart[r,s,ls,ld+1,y]-sum{lhlh in DAILYTIMEBRACKET:lh-lhlh<0} NetChargeWithinDay[r,s,ls,ld-1,lhlh,y])-StorageUpperLimit[r,s,y] <= 0;					
s.t. SC3_LowerLimit_EndOfDailyTimeBracketOfLastInstanceOfDayTypeInLastWeekConstraint{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR}:  0 <= (StorageLevelDayTypeFinish[r,s,ls,ld,y] - sum{lhlh in DAILYTIMEBRACKET:lh-lhlh<0} NetChargeWithinDay[r,s,ls,ld,lhlh,y])-StorageLowerLimit[r,s,y];																										
s.t. SC3_UpperLimit_EndOfDailyTimeBracketOfLastInstanceOfDayTypeInLastWeekConstraint{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR}:  (StorageLevelDayTypeFinish[r,s,ls,ld,y] - sum{lhlh in DAILYTIMEBRACKET:lh-lhlh<0} NetChargeWithinDay[r,s,ls,ld,lhlh,y])-StorageUpperLimit[r,s,y] <= 0;
s.t. SC4_LowerLimit_BeginningOfDailyTimeBracketOfFirstInstanceOfDayTypeInLastWeekConstraint{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR}: 	0 <= if ld > min{ldld in DAYTYPE} min(ldld) then (StorageLevelDayTypeFinish[r,s,ls,ld-1,y]+sum{lhlh in DAILYTIMEBRACKET:lh-lhlh>0} NetChargeWithinDay[r,s,ls,ld,lhlh,y])-StorageLowerLimit[r,s,y];
s.t. SC4_UpperLimit_BeginningOfDailyTimeBracketOfFirstInstanceOfDayTypeInLastWeekConstraint{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR}: if ld > min{ldld in DAYTYPE} min(ldld) then (StorageLevelDayTypeFinish[r,s,ls,ld-1,y]+sum{lhlh in DAILYTIMEBRACKET:lh-lhlh>0} NetChargeWithinDay[r,s,ls,ld,lhlh,y])-StorageUpperLimit[r,s,y] <= 0;										
s.t. SC5_MaxChargeConstraint{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR}: RateOfStorageCharge[r,s,ls,ld,lh,y] <= StorageMaxChargeRate[r,s];
s.t. SC6_MaxDischargeConstraint{r in REGION, s in STORAGE, ls in SEASON, ld in DAYTYPE, lh in DAILYTIMEBRACKET, y in YEAR}: RateOfStorageDischarge[r,s,ls,ld,lh,y] <= StorageMaxDischargeRate[r,s];
#
#########		Storage Investments				#############
#
s.t. SI1_StorageUpperLimit{r in REGION, s in STORAGE, y in YEAR}: AccumulatedNewStorageCapacity[r,s,y]+ResidualStorageCapacity[r,s,y] = StorageUpperLimit[r,s,y];
s.t. SI2_StorageLowerLimit{r in REGION, s in STORAGE, y in YEAR}: MinStorageCharge[r,s,y]*StorageUpperLimit[r,s,y] = StorageLowerLimit[r,s,y];
s.t. SI3_TotalNewStorage{r in REGION, s in STORAGE, y in YEAR}: sum{yy in YEAR: y-yy < OperationalLifeStorage[r,s] && y-yy>=0} NewStorageCapacity[r,s,yy]=AccumulatedNewStorageCapacity[r,s,y];
s.t. SI4_UndiscountedCapitalInvestmentStorage{r in REGION, s in STORAGE, y in YEAR}: CapitalCostStorage[r,s,y] * NewStorageCapacity[r,s,y] = CapitalInvestmentStorage[r,s,y];
s.t. SI5_DiscountingCapitalInvestmentStorage{r in REGION, s in STORAGE, y in YEAR}: CapitalInvestmentStorage[r,s,y]/((1+DiscountRateStorage[r,s])^(y-min{yy in YEAR} min(yy))) = DiscountedCapitalInvestmentStorage[r,s,y];
s.t. SI6_SalvageValueStorageAtEndOfPeriod1{r in REGION, s in STORAGE, y in YEAR: (y+OperationalLifeStorage[r,s]-1) <= (max{yy in YEAR} max(yy))}: 0 = SalvageValueStorage[r,s,y];
s.t. SI7_SalvageValueStorageAtEndOfPeriod2{r in REGION, s in STORAGE, y in YEAR: (DepreciationMethod[r]=1 && (y+OperationalLifeStorage[r,s]-1) > (max{yy in YEAR} max(yy)) && DiscountRateStorage[r,s]=0) || (DepreciationMethod[r]=2 && (y+OperationalLifeStorage[r,s]-1) > (max{yy in YEAR} max(yy)))}: CapitalInvestmentStorage[r,s,y]*(1-(max{yy in YEAR} max(yy) - y+1)/OperationalLifeStorage[r,s]) = SalvageValueStorage[r,s,y];
s.t. SI8_SalvageValueStorageAtEndOfPeriod3{r in REGION, s in STORAGE, y in YEAR: DepreciationMethod[r]=1 && (y+OperationalLifeStorage[r,s]-1) > (max{yy in YEAR} max(yy)) && DiscountRateStorage[r,s]>0}: CapitalInvestmentStorage[r,s,y]*(1-(((1+DiscountRateStorage[r,s])^(max{yy in YEAR} max(yy) - y+1)-1)/((1+DiscountRateStorage[r,s])^OperationalLifeStorage[r,s]-1))) = SalvageValueStorage[r,s,y];
s.t. SI9_SalvageValueStorageDiscountedToStartYear{r in REGION, s in STORAGE, y in YEAR}: SalvageValueStorage[r,s,y]/((1+DiscountRateStorage[r,s])^(max{yy in YEAR} max(yy)-min{yy in YEAR} min(yy)+1)) = DiscountedSalvageValueStorage[r,s,y];
s.t. SI10_TotalDiscountedCostByStorage{r in REGION, s in STORAGE, y in YEAR}: DiscountedCapitalInvestmentStorage[r,s,y]-DiscountedSalvageValueStorage[r,s,y] = TotalDiscountedStorageCost[r,s,y];
#
#########       	Capital Costs 		     	#############
#
s.t. CC1_UndiscountedCapitalInvestment{r in REGION, t in TECHNOLOGY, y in YEAR}: CapitalCost[r,t,y] * NewCapacity[r,t,y] = CapitalInvestment[r,t,y];
s.t. CC2_DiscountingCapitalInvestment{r in REGION, t in TECHNOLOGY, y in YEAR}: CapitalInvestment[r,t,y]/((1+DiscountRate[r,t])^(y-min{yy in YEAR} min(yy))) = DiscountedCapitalInvestment[r,t,y];
#
#########           Salvage Value            	#############
#
s.t. SV1_SalvageValueAtEndOfPeriod1{r in REGION, t in TECHNOLOGY, y in YEAR: (y + OperationalLife[r,t]-1) > (max{yy in YEAR} max(yy)) && DiscountRate[r,t]>0}: SalvageValue[r,t,y] = CapitalCost[r,t,y]*NewCapacity[r,t,y]*(1-(((1+DiscountRate[r,t])^(max{yy in YEAR} max(yy) - y+1)-1)/((1+DiscountRate[r,t])^OperationalLife[r,t]-1)));
s.t. SV2_SalvageValueAtEndOfPeriod2{r in REGION, t in TECHNOLOGY, y in YEAR: (y + OperationalLife[r,t]-1) > (max{yy in YEAR} max(yy)) && DiscountRate[r,t]=0}: SalvageValue[r,t,y] = CapitalCost[r,t,y]*NewCapacity[r,t,y]*(1-(max{yy in YEAR} max(yy) - y+1)/OperationalLife[r,t]);
s.t. SV3_SalvageValueAtEndOfPeriod3{r in REGION, t in TECHNOLOGY, y in YEAR: (y + OperationalLife[r,t]-1) <= (max{yy in YEAR} max(yy))}: SalvageValue[r,t,y] = 0;
s.t. SV4_SalvageValueDiscountedToStartYear{r in REGION, t in TECHNOLOGY, y in YEAR}: DiscountedSalvageValue[r,t,y] = SalvageValue[r,t,y]/((1+DiscountRate[r,t])^(1+max{yy in YEAR} max(yy)-min{yy in YEAR} min(yy)));
#
#########        	Operating Costs 		 	#############
#
s.t. OC1_OperatingCostsVariable{r in REGION, t in TECHNOLOGY, l in TIMESLICE, y in YEAR}: sum{m in MODE_OF_OPERATION} TotalAnnualTechnologyActivityByMode[r,t,m,y]*VariableCost[r,t,m,y] = AnnualVariableOperatingCost[r,t,y];
s.t. OC2_OperatingCostsFixedAnnual{r in REGION, t in TECHNOLOGY, y in YEAR}: TotalCapacityAnnual[r,t,y]*FixedCost[r,t,y] = AnnualFixedOperatingCost[r,t,y];
s.t. OC3_OperatingCostsTotalAnnual{r in REGION, t in TECHNOLOGY, y in YEAR}: AnnualFixedOperatingCost[r,t,y]+AnnualVariableOperatingCost[r,t,y] = OperatingCost[r,t,y];
s.t. OC4_DiscountedOperatingCostsTotalAnnual{r in REGION, t in TECHNOLOGY, y in YEAR}: OperatingCost[r,t,y]/((1+DiscountRate[r,t])^(y-min{yy in YEAR} min(yy)+0.5)) = DiscountedOperatingCost[r,t,y];
#
#########       	Total Discounted Costs	 	#############
#
s.t. TDC1_TotalDiscountedCostByTechnology{r in REGION, t in TECHNOLOGY, y in YEAR}: DiscountedOperatingCost[r,t,y]+DiscountedCapitalInvestment[r,t,y]+DiscountedTechnologyEmissionsPenalty[r,t,y]-DiscountedSalvageValue[r,t,y] = TotalDiscountedCostByTechnology[r,t,y];
s.t. TDC2_TotalDiscountedCost{r in REGION, y in YEAR}: sum{t in TECHNOLOGY} TotalDiscountedCostByTechnology[r,t,y]+sum{s in STORAGE} TotalDiscountedStorageCost[r,s,y] = TotalDiscountedCost[r,y];
#
#########      		Total Capacity Constraints 	##############
#
s.t. TCC1_TotalAnnualMaxCapacityConstraint{r in REGION, t in TECHNOLOGY, y in YEAR}: TotalCapacityAnnual[r,t,y] <= TotalAnnualMaxCapacity[r,t,y];
s.t. TCC2_TotalAnnualMinCapacityConstraint{r in REGION, t in TECHNOLOGY, y in YEAR: TotalAnnualMinCapacity[r,t,y]>0}: TotalCapacityAnnual[r,t,y] >= TotalAnnualMinCapacity[r,t,y];
#
#########    		New Capacity Constraints  	##############
#
s.t. NCC1_TotalAnnualMaxNewCapacityConstraint{r in REGION, t in TECHNOLOGY, y in YEAR}: NewCapacity[r,t,y] <= TotalAnnualMaxCapacityInvestment[r,t,y];
s.t. NCC2_TotalAnnualMinNewCapacityConstraint{r in REGION, t in TECHNOLOGY, y in YEAR: TotalAnnualMinCapacityInvestment[r,t,y]>0}: NewCapacity[r,t,y] >= TotalAnnualMinCapacityInvestment[r,t,y];
#
#########   		Annual Activity Constraints	##############
#
s.t. AAC1_TotalAnnualTechnologyActivity{r in REGION, t in TECHNOLOGY, y in YEAR}: sum{l in TIMESLICE} RateOfTotalActivity[r,t,l,y]*YearSplit[l,y] = TotalTechnologyAnnualActivity[r,t,y];
s.t. AAC2_TotalAnnualTechnologyActivityUpperLimit{r in REGION, t in TECHNOLOGY, y in YEAR}: TotalTechnologyAnnualActivity[r,t,y] <= TotalTechnologyAnnualActivityUpperLimit[r,t,y] ;
s.t. AAC3_TotalAnnualTechnologyActivityLowerLimit{r in REGION, t in TECHNOLOGY, y in YEAR: TotalTechnologyAnnualActivityLowerLimit[r,t,y]>0}: TotalTechnologyAnnualActivity[r,t,y] >= TotalTechnologyAnnualActivityLowerLimit[r,t,y] ;
#
#########    		Total Activity Constraints 	##############
#
s.t. TAC1_TotalModelHorizonTechnologyActivity{r in REGION, t in TECHNOLOGY}: sum{y in YEAR} TotalTechnologyAnnualActivity[r,t,y] = TotalTechnologyModelPeriodActivity[r,t];
s.t. TAC2_TotalModelHorizonTechnologyActivityUpperLimit{r in REGION, t in TECHNOLOGY, y in YEAR}: TotalTechnologyModelPeriodActivity[r,t] <= TotalTechnologyModelPeriodActivityUpperLimit[r,t] ;
s.t. TAC3_TotalModelHorizenTechnologyActivityLowerLimit{r in REGION, t in TECHNOLOGY, y in YEAR: TotalTechnologyModelPeriodActivityLowerLimit[r,t]>0}: TotalTechnologyModelPeriodActivity[r,t] >= TotalTechnologyModelPeriodActivityLowerLimit[r,t] ;
#
#########   		Reserve Margin Constraint	############## NTS: Should change demand for production
#
s.t. RM1_ReserveMargin_TechologiesIncluded_In_Activity_Units{r in REGION, l in TIMESLICE, y in YEAR}: sum {t in TECHNOLOGY} TotalCapacityAnnual[r,t,y] * ReserveMarginTagTechnology[r,t,y] * CapacityToActivityUnit[r,t]	 = 	TotalCapacityInReserveMargin[r,y];
s.t. RM2_ReserveMargin_FuelsIncluded{r in REGION, l in TIMESLICE, y in YEAR}: sum {f in FUEL} RateOfProduction[r,l,f,y] * ReserveMarginTagFuel[r,f,y] = DemandNeedingReserveMargin[r,l,y];
s.t. RM3_ReserveMargin_Constraint{r in REGION, l in TIMESLICE, y in YEAR}: DemandNeedingReserveMargin[r,l,y] * ReserveMargin[r,y]<= TotalCapacityInReserveMargin[r,y];
#
#########   		RE Production Target		############## NTS: Should change demand for production
#
s.t. RE1_FuelProductionByTechnologyAnnual{r in REGION, t in TECHNOLOGY, f in FUEL, y in YEAR}: sum{l in TIMESLICE} ProductionByTechnology[r,l,t,f,y] = ProductionByTechnologyAnnual[r,t,f,y];
s.t. RE2_TechIncluded{r in REGION, y in YEAR}: sum{t in TECHNOLOGY, f in FUEL} ProductionByTechnologyAnnual[r,t,f,y]*RETagTechnology[r,t,y] = TotalREProductionAnnual[r,y];
s.t. RE3_FuelIncluded{r in REGION, y in YEAR}: sum{l in TIMESLICE, f in FUEL} RateOfDemand[r,l,f,y]*YearSplit[l,y]*RETagFuel[r,f,y] = RETotalDemandOfTargetFuelAnnual[r,y]; 
s.t. RE4_EnergyConstraint{r in REGION, y in YEAR}:REMinProductionTarget[r,y]*RETotalDemandOfTargetFuelAnnual[r,y] <= TotalREProductionAnnual[r,y];
s.t. RE5_FuelUseByTechnologyAnnual{r in REGION, t in TECHNOLOGY, f in FUEL, y in YEAR}: sum{l in TIMESLICE} RateOfUseByTechnology[r,l,t,f,y]*YearSplit[l,y] = UseByTechnologyAnnual[r,t,f,y];
#
#########   		Emissions Accounting		##############
#
s.t. E1_AnnualEmissionProductionByMode{r in REGION, t in TECHNOLOGY, e in EMISSION, m in MODE_OF_OPERATION, y in YEAR: EmissionActivityRatio[r,t,e,m,y]<>0}: EmissionActivityRatio[r,t,e,m,y]*TotalAnnualTechnologyActivityByMode[r,t,m,y]=AnnualTechnologyEmissionByMode[r,t,e,m,y];
s.t. E2_AnnualEmissionProduction{r in REGION, t in TECHNOLOGY, e in EMISSION, y in YEAR}: sum{m in MODE_OF_OPERATION} AnnualTechnologyEmissionByMode[r,t,e,m,y] = AnnualTechnologyEmission[r,t,e,y];
s.t. E3_EmissionsPenaltyByTechAndEmission{r in REGION, t in TECHNOLOGY, e in EMISSION, y in YEAR}: AnnualTechnologyEmission[r,t,e,y]*EmissionsPenalty[r,e,y] = AnnualTechnologyEmissionPenaltyByEmission[r,t,e,y];
s.t. E4_EmissionsPenaltyByTechnology{r in REGION, t in TECHNOLOGY, y in YEAR}: sum{e in EMISSION} AnnualTechnologyEmissionPenaltyByEmission[r,t,e,y] = AnnualTechnologyEmissionsPenalty[r,t,y];
s.t. E5_DiscountedEmissionsPenaltyByTechnology{r in REGION, t in TECHNOLOGY, y in YEAR}: AnnualTechnologyEmissionsPenalty[r,t,y]/((1+DiscountRate[r,t])^(y-min{yy in YEAR} min(yy)+0.5)) = DiscountedTechnologyEmissionsPenalty[r,t,y];
s.t. E6_EmissionsAccounting1{r in REGION, e in EMISSION, y in YEAR}: sum{t in TECHNOLOGY} AnnualTechnologyEmission[r,t,e,y] = AnnualEmissions[r,e,y];
s.t. E7_EmissionsAccounting2{r in REGION, e in EMISSION}: sum{y in YEAR} AnnualEmissions[r,e,y] = ModelPeriodEmissions[r,e]- ModelPeriodExogenousEmission[r,e];
s.t. E8_AnnualEmissionsLimit{r in REGION, e in EMISSION, y in YEAR}: AnnualEmissions[r,e,y]+AnnualExogenousEmission[r,e,y] <= AnnualEmissionLimit[r,e,y];
s.t. E9_ModelPeriodEmissionsLimit{r in REGION, e in EMISSION}: ModelPeriodEmissions[r,e] <= ModelPeriodEmissionLimit[r,e] ;





##################################################################################################################################
# The following will solve the model and write interesting results
# in the file results.m (Matlab/Octave format).
# 
##################################################################################################################################



solve;






#############################################################
# print results in the Matlab or Octave file results.m 
#############################################################

printf "clear;\n\n" > "results.m";




printf "year =[\n" >> "results.m";
for {y in YEAR} printf "%d \n", y>> "results.m";
printf "];\n\n" >> "results.m";




############
#Costs

printf "cost=%f;\n", cost >> "results.m";

printf "cost_subvention=%f;\n", cost_subvention>> "results.m";





############
#RL

printf "RateOfDemand =[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, l in TIMESLICE, f in FUEL: f='RL'}
printf "%f   ", RateOfDemand[r,l,f,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";



printf "Demand =[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, l in TIMESLICE, f in FUEL: f='RL'}
printf "%f   ", Demand[r,l,f,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";




printf "SpecifiedAnnualDemand=[\n" >> "results.m";
for {r in REGION, f in FUEL, y in YEAR: f= 'RL'} printf "%f \n", SpecifiedAnnualDemand[r,f,y]>> "results.m";
printf "];\n\n" >> "results.m";


printf "SpecifiedDemandProfile=[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, l in TIMESLICE, f in FUEL: f='RL'}
printf "%f   ", SpecifiedDemandProfile[r,f,l,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";





############
#RL1


printf "ProductionByTechnologyAnnualRL1=[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, t in TECHNOLOGY, f in FUEL: t='RL1' and f = 'RL'} 
printf "%f    ", ProductionByTechnologyAnnual[r,t,f,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";


printf "NewCapacityRL1 =[\n" >> "results.m";
for {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL1'} printf "%f \n", NewCapacity[r,t,y]>> "results.m";
printf "];\n\n" >> "results.m";


printf "AccumulatedNewCapacityRL1 =[\n" >> "results.m";
for {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL1'} printf "%f \n", AccumulatedNewCapacity [r,t,y]>> "results.m";
printf "];\n\n" >> "results.m";


printf "TotalCapacityAnnualRL1 =[\n" >> "results.m";
for {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL1'} printf "%f \n", TotalCapacityAnnual [r,t,y]>> "results.m";
printf "];\n\n" >> "results.m";


printf "OperatingCostRL1=[\n" >> "results.m";
for {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL1'} printf "%f \n", OperatingCost[r,t,y]>> "results.m";
printf "];\n\n" >> "results.m";


printf "AnnualFixedOperatingCostRL1=[\n" >> "results.m";
for {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL1'} printf "%f \n", AnnualFixedOperatingCost[r,t,y]>> "results.m";
printf "];\n\n" >> "results.m";


printf "AnnualVariableOperatingCostRL1=[\n" >> "results.m";
for {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL1'} printf "%f \n", AnnualVariableOperatingCost[r,t,y]>> "results.m";
printf "];\n\n" >> "results.m";


printf "CapitalInvestmentRL1=[\n" >> "results.m";
for {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL1'} printf "%f \n", CapitalInvestment[r,t,y]>> "results.m";
printf "];\n\n" >> "results.m";


printf "TotalDiscountedCostRL1 =[\n" >> "results.m";
for {y in YEAR}
printf "%f   \n", TotalDiscountedCostByTechnology['UTOPIA','RL1',y]>> "results.m";
printf "];\n\n" >> "results.m";







############
#RL2

printf "ProductionByTechnologyAnnualRL2=[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, t in TECHNOLOGY, f in FUEL: t='RL2' and f = 'RL'} 
printf "%f    ", ProductionByTechnologyAnnual[r,t,f,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";



printf "NewCapacityRL2 =[\n" >> "results.m";
for {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL2'} printf "%f \n", NewCapacity[r,t,y]>> "results.m";
printf "];\n\n" >> "results.m";


printf "AccumulatedNewCapacityRL2 =[\n" >> "results.m";
for {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL2'} printf "%f \n", AccumulatedNewCapacity [r,t,y]>> "results.m";
printf "];\n\n" >> "results.m";


printf "TotalCapacityAnnualRL2 =[\n" >> "results.m";
for {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL2'} printf "%f \n", TotalCapacityAnnual [r,t,y]>> "results.m";
printf "];\n\n" >> "results.m";


printf "OperatingCostRL2=[\n" >> "results.m";
for {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL2'} printf "%f \n", OperatingCost[r,t,y]>> "results.m";
printf "];\n\n" >> "results.m";


printf "AnnualFixedOperatingCostRL2=[\n" >> "results.m";
for {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL2'} printf "%f \n", AnnualFixedOperatingCost[r,t,y]>> "results.m";
printf "];\n\n" >> "results.m";


printf "AnnualVariableOperatingCostRL2=[\n" >> "results.m";
for {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL2'} printf "%f \n", AnnualVariableOperatingCost[r,t,y]>> "results.m";
printf "];\n\n" >> "results.m";


printf "CapitalInvestmentRL2=[\n" >> "results.m";
for {r in REGION, t in TECHNOLOGY, y in YEAR: t = 'RL2'} printf "%f \n", CapitalInvestment[r,t,y]>> "results.m";
printf "];\n\n" >> "results.m";




printf "TotalDiscountedCostRL2 =[\n" >> "results.m";
for {y in YEAR}
printf "%f   \n", TotalDiscountedCostByTechnology['UTOPIA','RL2',y]>> "results.m";
printf "];\n\n" >> "results.m";








############
#E01 + HCO
 
printf "ProductionByTechnologyAnnualE01=[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, t in TECHNOLOGY, f in FUEL: t='E01' and f = 'ELC'} 
printf "%f    ", ProductionByTechnologyAnnual[r,t,f,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";


printf "TotalDiscountedCostE01 =[\n" >> "results.m";
for {y in YEAR}
printf "%f   \n", 
TotalDiscountedCostByTechnology['UTOPIA','E01',y] >> "results.m";
printf "];\n\n" >> "results.m";


printf "ProductionHCO=[\n" >> "results.m";
for {y in YEAR}
{
printf "%f    ", (sum {r in REGION, l in TIMESLICE, f in FUEL : f = 'HCO'} Production[r,l,f,y]) >> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";


printf "TotalDiscountedCostIMPHCO1 =[\n" >> "results.m";
for {y in YEAR}
printf "%f   \n", 
TotalDiscountedCostByTechnology['UTOPIA','IMPHCO1',y] >> "results.m";
printf "];\n\n" >> "results.m";












############
#E21 + URN
 
printf "ProductionByTechnologyAnnualE21=[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, t in TECHNOLOGY, f in FUEL: t='E21' and f = 'ELC'} 
printf "%f    ", ProductionByTechnologyAnnual[r,t,f,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";


printf "TotalDiscountedCostE21 =[\n" >> "results.m";
for {y in YEAR}
printf "%f   \n", 
TotalDiscountedCostByTechnology['UTOPIA','E21',y] >> "results.m";
printf "];\n\n" >> "results.m";


printf "ProductionURN=[\n" >> "results.m";
for {y in YEAR}
{
printf "%f    ", (sum {r in REGION, l in TIMESLICE, f in FUEL : f = 'URN'} Production[r,l,f,y]) >> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";


printf "TotalDiscountedCostIMPURN1 =[\n" >> "results.m";
for {y in YEAR}
printf "%f   \n", 
TotalDiscountedCostByTechnology['UTOPIA','IMPURN1',y] >> "results.m";
printf "];\n\n" >> "results.m";






############
#E31 +RIV


printf "ProductionByTechnologyAnnualE31=[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, t in TECHNOLOGY, f in FUEL: t='E31' and f = 'ELC'} 
printf "%f    ", ProductionByTechnologyAnnual[r,t,f,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";


printf "TotalDiscountedCostE31 =[\n" >> "results.m";
for {y in YEAR}
printf "%f   \n", 
TotalDiscountedCostByTechnology['UTOPIA','E31',y] >> "results.m";
printf "];\n\n" >> "results.m";




printf "TotalDiscountedCostRIV =[\n" >> "results.m";
for {y in YEAR}
printf "%f   \n", TotalDiscountedCostByTechnology['UTOPIA','RIV',y]>> "results.m";
printf "];\n\n" >> "results.m";







############
#E51


printf "ProductionByTechnologyAnnualE51=[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, t in TECHNOLOGY, f in FUEL: t='E51' and f = 'ELC'} 
printf "%f    ", ProductionByTechnologyAnnual[r,t,f,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";


printf "TotalDiscountedCostE51 =[\n" >> "results.m";
for {y in YEAR}
printf "%f   \n", 
TotalDiscountedCostByTechnology['UTOPIA','E51',y] >> "results.m";
printf "];\n\n" >> "results.m";





############
#E70 + DLS +TXD+RHO
 
printf "ProductionByTechnologyAnnualE70=[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, t in TECHNOLOGY, f in FUEL: t='E70' and f = 'ELC'} 
printf "%f    ", ProductionByTechnologyAnnual[r,t,f,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";


printf "TotalDiscountedCostE70 =[\n" >> "results.m";
for {y in YEAR}
printf "%f   \n", 
TotalDiscountedCostByTechnology['UTOPIA','E70',y] >> "results.m";
printf "];\n\n" >> "results.m";


printf "ProductionDSL=[\n" >> "results.m";
for {y in YEAR}
{
printf "%f    ", (sum {r in REGION, l in TIMESLICE, f in FUEL : f = 'DSL'} Production[r,l,f,y]) >> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";


printf "TotalDiscountedCostIMPDSL1 =[\n" >> "results.m";
for {y in YEAR}
printf "%f   \n", 
TotalDiscountedCostByTechnology['UTOPIA','IMPDSL1',y] >> "results.m";
printf "];\n\n" >> "results.m";




printf "ProductionByTechnologyAnnualTXD=[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, t in TECHNOLOGY, f in FUEL: t='TXD' and f = 'TX'} 
printf "%f    ", ProductionByTechnologyAnnual[r,t,f,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";


printf "ProductionByTechnologyAnnualRHO=[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, t in TECHNOLOGY, f in FUEL: t='RHO' and f = 'RH'} 
printf "%f    ", ProductionByTechnologyAnnual[r,t,f,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";









############
#ELC


printf "ProductionByTechnologyAnnualELC=[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, t in TECHNOLOGY, f in FUEL: f = 'ELC'} 
printf "%f    ", ProductionByTechnologyAnnual[r,t,f,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";


printf "ProductionELC=[\n" >> "results.m";
for {y in YEAR}
{
printf "%f    ", (sum {r in REGION, l in TIMESLICE, f in FUEL : f = 'ELC'} Production[r,l,f,y]) >> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";




printf "ProductionByTechnologyAnnualTXE=[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, t in TECHNOLOGY, f in FUEL: t='TXE' and f = 'ELC'} 
printf "%f    ", ProductionByTechnologyAnnual[r,t,f,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";


printf "ProductionByTechnologyAnnualRHE=[\n" >> "results.m";
for {y in YEAR}
{for {r in REGION, t in TECHNOLOGY, f in FUEL: t='RHE' and f = 'ELC'} 
printf "%f    ", ProductionByTechnologyAnnual[r,t,f,y]>> "results.m";
printf "\n" >> "results.m";
}
printf "];\n\n" >> "results.m";






############
#plots



printf "plot(year,TotalCapacityAnnualRL1,year,TotalCapacityAnnualRL2);\n" >> "results.m";










end;


